<?php

/**

 * Plugin Name: CodeFusionOnline Optimizer

 * Plugin URI: https://codefusiononline.com/

 * Description: Enhance your website's performance on tools like GTmetrix, Pingdom, and Google PageSpeed Insights by optimizing CSS and JavaScript files through merging and minification, compressing HTML, and applying additional speed improvement techniques. 

 * Version: 1.0

 * Author: codefusiononline

 * Author URI: https://codefusiononline.com/

 * Text Domain: codefusiononline

 * Requires at least: 5.6

 * Requires PHP: 7.3

 * License: GPL3 / https://wordpress.org/about/license/

*/



# Exit if accessed directly				

if (!defined('ABSPATH')){ exit(); }	


require plugin_dir_path(__FILE__) . 'update/plugin-update-checker.php';

use YahnisElsts\PluginUpdateChecker\v5\PucFactory;



$myUpdateChecker = PucFactory::buildUpdateChecker(

    'https://codefusiononline.com/updates/optimizer/codefusiononline.json', // JSON file URL

    __FILE__, // Plugin's main file path

    'CodeFImageOptimizer' // Plugin slug

);


# Invalidate OPCache for current file on WP 5.5+

if(function_exists('wp_opcache_invalidate') && stripos(__FILE__, '/codefonline.php') !== false) {

	wp_opcache_invalidate(__FILE__, true);

}



# info, variables, paths

if (!defined('codefonline_PDIR')) { define('codefonline_PDIR', __DIR__ . '/'); }  # /home/path/plugins/pluginname/

$codefonline_var_dir_path = codefonline_PDIR;                               		# /home/path/plugins/pluginname/

$codefonline_var_file = codefonline_PDIR . 'codefonline.php';                       		# /home/path/plugins/pluginname/wpr.php

$codefonline_var_inc_dir = codefonline_PDIR . 'inc' . DIRECTORY_SEPARATOR;  		# /home/path/plugins/pluginname/inc/

$codefonline_var_inc_lib = codefonline_PDIR . 'libs' . DIRECTORY_SEPARATOR; 		# /home/path/plugins/pluginname/libs/

$codefonline_var_basename = plugin_basename($codefonline_var_file);                 # pluginname/wpr.php

$codefonline_var_url_path = plugins_url(dirname($codefonline_var_basename)) . '/';  # https://example.com/wp-content/plugins/pluginname/

$codefonline_var_plugin_version = get_file_data($codefonline_var_file, array('Version' => 'Version'), false)['Version'];



# global functions for backend, frontend, ajax, etc

require_once($codefonline_var_inc_dir . 'common.php');

require_once($codefonline_var_inc_dir . 'updates.php');



# wp-cli support

if (defined('WP_CLI') && WP_CLI) {

	require_once($codefonline_var_inc_dir . 'wp-cli.php');

}



# get all options from database

$codefonline_settings = codefonline_get_settings();



# site url, domain name

$codefonline_urls = array('wp_site_url'=>trailingslashit(site_url()), 'wp_domain'=>codefonline_get_domain());





# only on backend

if(is_admin()) {

	

	# admin functionality

	require_once($codefonline_var_inc_dir . 'admin.php');

	require_once($codefonline_var_inc_dir . 'serverinfo.php');



	# both backend and frontend, as long as user can manage options

	add_action('admin_bar_menu', 'codefonline_admintoolbar', 100);

	add_action('init', 'codefonline_process_cache_purge_request');

		

	# do admin stuff, as long as user can manage options

	add_action('admin_init', 'codefonline_save_settings');

	add_action('admin_init', 'codefonline_check_minimum_requirements');

	add_action('admin_init', 'codefonline_check_misconfiguration');

	add_action('admin_init', 'codefonline_update_changes');

	add_action('admin_enqueue_scripts', 'codefonline_add_admin_jscss');

	add_action('admin_menu', 'codefonline_add_admin_menu');

	add_action('admin_notices', 'codefonline_show_admin_notice_from_transient');

	add_action('wp_ajax_codefonline_get_logs', 'codefonline_get_logs_callback');

		

	# purge everything

	add_action('switch_theme', 'codefonline_purge_all');

	add_action('customize_save', 'codefonline_purge_all');

	add_action('avada_clear_dynamic_css_cache', 'codefonline_purge_all');

	add_action('upgrader_process_complete', 'codefonline_purge_all');

	add_action('update_option_theme_mods_' . get_option('stylesheet'), 'codefonline_purge_all');

	

}







# frontend only, any user permissions

if(!is_admin()) {

	

	# frontend functionality

	require_once($codefonline_var_inc_dir . 'frontend.php');

	

	# both back and front, as long as the option is enabled

	add_action('init', 'codefonline_disable_emojis');

	

	# both backend and frontend, as long as user can manage options

	add_action('admin_bar_menu', 'codefonline_admintoolbar', 100);

	add_action('init', 'codefonline_process_cache_purge_request');

	

	# load after all plugins

	add_action( 'plugins_loaded', 'codefonline_loader' );

	function codefonline_loader() {

		$active_plugins = apply_filters('active_plugins', get_option('active_plugins'));

		if (in_array('wp-super-cache/wp-cache.php', $active_plugins)) {

			add_filter( 'wpsupercache_buffer', 'codefonline_process_page' );          # WP-Super-Cache

		} else if (in_array('w3-total-cache/w3-total-cache.php', $active_plugins)) {

			add_filter( 'w3tc_process_content', 'codefonline_process_page' );         # W3 Total Cache

		} else if (in_array('wp-rocket/wp-rocket.php', $active_plugins)) {

			add_filter( 'rocket_buffer', 'codefonline_process_page' );                # WP Rocket

		} else if (in_array('litespeed-cache/litespeed-cache.php', $active_plugins)) {

			add_filter( 'litespeed_buffer_before', 'codefonline_process_page' );      # LiteSpeed Cache

		} else if (in_array('cache-enabler/cache-enabler.php', $active_plugins)) {

			add_filter( 'cache_enabler_page_contents_before_store', 'codefonline_process_page' );   # Cache Enabler

		} else {

			if (!defined('codefonline_HOOK_INTO')) { define('codefonline_HOOK_INTO', 'setup_theme'); }

			add_action(constant("codefonline_HOOK_INTO"), 'codefonline_start_buffer', 999999);

		}

	}

		

}



