<?php



# Exit if accessed directly				

if (!defined('ABSPATH')){ exit(); }	



# check for minimum requirements and prevent activation or disable if not fully compatible

function codefonline_check_minimum_requirements() {

	if(current_user_can('manage_options')) {

		# defaults

		$error = '';

		# php version requirementscodefusiononline

		if (version_compare( PHP_VERSION, '5.6', '<' )) { 

			$error = __( 'codefonline requires PHP 5.6 or higher. You’re still on', 'codefusiononline' ) .' '. PHP_VERSION; 

		}

		# php extension requirements	

		if (!extension_loaded('mbstring')) { 

			$error = __( 'codefonline requires the PHP mbstring module to be installed on the server.', 'codefusiononline' ); 

		}

		

		# wp version requirements

		if ( version_compare( $GLOBALS['wp_version'], '4.9', '<' ) ) {

			$error = __( 'codefonline requires WP 4.9 or higher. You’re still on', 'codefusiononline' ) .' '. $GLOBALS['wp_version']; 

		}

		

		# check cache directory

		$ch_info = codefonline_get_cache_location();

		if(isset($ch_info['ch_url'])  && !empty($ch_info['ch_url']) && isset($ch_info['ch_dir']) && !empty($ch_info['ch_dir'])) {

			if(is_dir($ch_info['ch_dir']) && !is_writable($ch_info['ch_dir'])) {

				$error = __( 'codefonline needs writing permissions on ', 'codefusiononline' ). ' ['.$ch_info['ch_dir'].']';

			}

		}		

		

		# deactivate plugin forcefully

		global $codefonline_var_basename;

		if ((is_plugin_active($codefonline_var_basename) && !empty($error)) || !empty($error)) { 

		if (isset($_GET['activate'])) { unset($_GET['activate']); }

			deactivate_plugins($codefonline_var_basename); 

			add_settings_error( 'codefonline_admin_notice', 'codefonline_admin_notice', $error, 'success' );

		}

		

	}

}





# check for soft errors and misconfiguration

function codefonline_check_misconfiguration() {

	try {

				

		# plugin version

		global $codefonline_var_plugin_version;

		if(is_null($codefonline_var_plugin_version)) { return false; }

				

		# if no database version, regenerate

		$plugin_meta = get_option('codefonline_plugin_meta');

		if($plugin_meta === false) {

			

			# startup routines

			codefonline_plugin_deactivate();

			codefonline_plugin_activate();

			

			# save

			update_option('codefonline_plugin_meta', json_encode(array('dbv'=>0)) );

		}

		

		# updates

		if($plugin_meta !== false) {

			

			# future updates

			$meta = json_decode($plugin_meta, true);

			$previous_version = $meta['dbv'];

			if($codefonline_var_plugin_version != $previous_version) {

				# startup routines

				codefonline_plugin_deactivate();

				codefonline_plugin_activate();

				

				# save

				update_option('codefonline_plugin_meta', json_encode(array('dbv'=>$codefonline_var_plugin_version)) );

				

			}

			

		}

		

		# check if our tables exist, and do maintenance once a day

		$codefonline_table_checker = get_transient('codefonline_table_checker');

		$codefonline_table_checker = false;

		if ($codefonline_table_checker === false) {

			

			# test if at least one table exists

			global $wpdb;

			if(!is_null($wpdb)) {

				$sqla_table_name = $wpdb->prefix . 'codefonline_cache';

				if (!$wpdb->get_var($wpdb->prepare("SHOW TABLES LIKE %s", $sqla_table_name)) === $sqla_table_name) {

					codefonline_plugin_activate();								

				}

			}

			

			# daily maintenance

			try {

				

				if(!is_null($wpdb)) { 

					

					# limit cache table to 20k records

					$lim = 20000;

					$res = $wpdb->get_row("SELECT MAX(id) as maxid FROM ".$wpdb->prefix."codefonline_cache");

					if(isset($res->maxid) && intval($res->maxid) > $lim) {

						$wpdb->query($wpdb->prepare("DELETE FROM ".$wpdb->prefix."codefonline_cache WHERE id < %d LIMIT 1", (intval($res->maxid) - $lim)));

					}

					

					# limit logs table to 500 records

					$lim = 500;

					$res = $wpdb->get_row("SELECT MAX(id) as maxid FROM ".$wpdb->prefix."codefonline_logs");

					if(isset($res->maxid) && intval($res->maxid) > $lim) {

						$wpdb->query($wpdb->prepare("DELETE FROM ".$wpdb->prefix."codefonline_logs WHERE id < %d LIMIT 1", (intval($res->maxid) - $lim)));

					}					

					

				}

				

			} catch (Exception $e) {

				error_log('Error: '.$e->getMessage(), 0);

			}

			

			

		}

		



	} catch (Exception $e) {

		error_log('Caught exception (codefonline_initialize_database): '.$e->getMessage(), 0);

	}

}









# save plugin settings on wp-admin

function codefonline_save_settings() {



	# save settings

	if(isset($_POST['codefonline_action']) && isset($_POST['codefonline_settings_nonce']) && $_POST['codefonline_action'] == 'save_settings') {

		

		if(!current_user_can('manage_options')) {

			wp_die( __('You do not have sufficient permissions to access this page.', 'codefusiononline'), __('Error:', 'codefusiononline'), array('response'=>200)); 

		}

		

		if(!wp_verify_nonce($_POST['codefonline_settings_nonce'], 'codefonline_settings_nonce')) {

			wp_die( __('Invalid nounce. Please refresh and try again.', 'codefusiononline'), __('Error:', 'codefusiononline'), array('response'=>200)); 

		}

		

		# update codefonline_settings in the global scope

		if(isset($_POST['codefonline_settings']) && is_array($_POST['codefonline_settings'])) {

			

			# sanitize recursively

			if(is_array($_POST['codefonline_settings'])) {

				foreach ($_POST['codefonline_settings'] as $group=>$arr) {

					if(is_array($arr)) {

						foreach ($arr as $k=>$v) {

							

							# only numeric, string or arrays allowed at this level

							if(!is_string($v) && !is_numeric($v) && !is_array($v)) { $_POST['codefonline_settings'][$group][$k] = ''; }

							

							# numeric fields, only positive integers allowed 

							if(is_numeric($v)) { $_POST['codefonline_settings'][$group][$k] = abs(intval($v)); }

							

							# sanitize text area content

							if(is_string($v)) { $_POST['codefonline_settings'][$group][$k] = strip_tags($v); }

							

							# clean cdn url

							if($group == 'cdn' && $k == 'url') { 

								$_POST['codefonline_settings'][$group][$k] = trim(trim(str_replace(array('http://', 'https://'), '', $v), '/'));

							}

		

						}

					}

				}

			}

			

			# get mandatory default exclusions

			global $codefonline_settings;

			$codefonline_settings = codefonline_get_default_settings($_POST['codefonline_settings']);

			

			# purge caches

			codefonline_purge_all();
			
			# save settings

			update_option('codefonline_settings', json_encode($codefonline_settings), false);

			add_settings_error( 'codefonline_admin_notice', 'codefonline_admin_notice', 'Settings saved successfully!', 'success' );

		

		} else {

			wp_die( __('Invalid data!', 'codefusiononline'), __('Error:', 'codefusiononline'), array('response'=>200)); 

		}

	}

}



# return checked, or empty for checkboxes in admin

function codefonline_get_settings_checkbox($value) {

	if($value == 1) { return 'checked'; }

	return '';

}



# return checked, or empty for checkboxes in admin

function codefonline_get_settings_radio($key, $value) {

	if($key == $value) { return 'checked'; }

	return '';

}





# add settings link on plugins listing page

add_filter("plugin_action_links_".$codefonline_var_basename, 'codefonline_min_settings_link' );

function codefonline_min_settings_link($links) {

	global $codefonline_var_basename;

	if (is_plugin_active($codefonline_var_basename)) { 

		$settings_link = '<a href="'.admin_url('admin.php?page=codefonline').'">Settings</a>'; 

		array_unshift($links, $settings_link); 

	}

return $links;

}





# Enqueue plugin UI CSS and JS files

function codefonline_add_admin_jscss($hook) {

	if(current_user_can('manage_options')) {

		if ('settings_page_codefonline' != $hook) { return; }

		global $codefonline_var_dir_path, $codefonline_var_url_path;

		

		# ui

		wp_enqueue_script( 'jquery-ui-core' );

		wp_enqueue_script( 'jquery-ui-accordion' );

		

		# js

		wp_enqueue_script('codefonline', $codefonline_var_url_path . 'assets/codefonline.js', array('jquery'), filemtime($codefonline_var_dir_path.'assets'. DIRECTORY_SEPARATOR .'codefonline.js'));

		# css

		wp_enqueue_style('codefonline', $codefonline_var_url_path . 'assets/codefonline.css', array(), filemtime($codefonline_var_dir_path.'assets'. DIRECTORY_SEPARATOR .'codefonline.css'));

		

	}

}





# create sidebar admin menu and add templates to admin

function codefonline_add_admin_menu() {

	if (current_user_can('manage_options')) {

		//add_options_page('codefonline Settings', 'CodeFusionOnline Optimizer', 'manage_options', 'codefonline', 'codefonline_add_settings_admin');
		
		// Add the main menu
        add_menu_page(
            'CodeFusionOnline Optimizer', // Page title
            'CodeFusionOnline',           // Menu title
            'manage_options',             // Capability
            'codefonline',           // Menu slug
            'codefonline_add_settings_admin',  // Callback function for the main menu
            'dashicons-admin-generic',    // Icon
            90                            // Position
        );

        // Add sub-menu for Critical CSS settings
        add_submenu_page(
            'codefonline',           // Parent slug
            'Critical CSS Settings',      // Page title
            'Critical CSS',               // Sub-menu title
            'manage_options',             // Capability
            'codefonline_critical_css',   // Sub-menu slug
            'codefonline_critical_css_settings'  // Callback function for the sub-menu
        );

	}

}





# print admin notices when needed (json)

function codefonline_show_admin_notice_from_transient() {

	if(current_user_can('manage_options')) {

		$inf = get_transient('codefonline_admin_notice_'.get_current_user_id());



		# show transient after the redirect

		if($inf != false && !empty($inf)) {

			$notices = json_decode($inf, true);

			if(!is_null($notices) && is_array($notices)){

				

				# consolidate messages

				$notices = array_unique($notices);

				if(count($notices) > 1) {

					$msg = '<div class="codefonline-info-list"><h3>codefonline</h3><ul>';

					foreach ($notices as $notice) { $msg.= "<li>$notice</li>"; }

					$msg.= '</ul></div>';

					add_settings_error( 'codefonline_admin_notice', 'codefonline_admin_notice', $msg, 'info' );

				} else {

					$msg = 'codefonline: '.implode(PHP_EOL, $notices);

					add_settings_error( 'codefonline_admin_notice', 'codefonline_admin_notice', $msg, 'info' );

				}

				

			}

			

			# delete

			delete_transient('codefonline_admin_notice_'.get_current_user_id());

			

		}

	}

}



# manage settings page

function codefonline_add_settings_admin() {

	

	# admin only

	if (!current_user_can('manage_options')) { 

		wp_die( __('You do not have sufficient permissions to access this page.'), __('Error:'), array('response'=>200)); 

	}



	# include admin html template

	global $codefonline_settings, $codefonline_var_dir_path;

	

	# admin html templates

	include($codefonline_var_dir_path . 'layout' . DIRECTORY_SEPARATOR . 'admin-layout.php');



}





# function to list all cache files on the status page (js ajax code)

function codefonline_get_logs_callback() {

		

	# must be able to cleanup cache

	if (!current_user_can('manage_options')) {

		wp_die( __('You do not have sufficient permissions to access this page.'), __('Error:'), array('response'=>200)); 

	}

	

	# must have

	if(!defined('WP_CONTENT_DIR')) { 

		wp_die( __('WP_CONTENT_DIR is undefined!'), __('Error:'), array('response'=>200)); 

	}

	

	# defaults

	global $wpdb;

	if(is_null($wpdb)) { 

		wp_die( __('Database error!'), __('Error:'), array('response'=>200)); 

	}

	

	# initialize log

	$log = '';

		

	# build css logs from database

	$results = $wpdb->get_results("SELECT date, msg FROM ".$wpdb->prefix."codefonline_logs ORDER BY id DESC LIMIT 500", 'ARRAY_A');

		

	# build log

	if(is_array($results)) {

		foreach (array_reverse($results, true) as $r) {

			$log.= 'PROCESSED ON - ' . date('r', $r['date']) . PHP_EOL;

			$log.= $r['msg'] .  PHP_EOL . PHP_EOL;

		}

	}

	

	# default message

	if(empty($log)) { $log = 'No logs generated yet.'; }



		# build info

		$result = array(

			'log' => $log,

			'success' => 'OK'

		);

		

		# return result

		header('Content-Type: application/json');

		echo json_encode($result);

		exit();

	

}





# run during activation

register_activation_hook($codefonline_var_file, 'codefonline_plugin_activate');

function codefonline_plugin_activate() {

		

	# defauls

	global $wpdb;

	if(is_null($wpdb)) { return false; }

	$charset_collate = $wpdb->get_charset_collate();

	$sqla_table_name = $wpdb->prefix . 'codefonline_cache';

	$sqlb_table_name = $wpdb->prefix . 'codefonline_logs';

		

	# create cache table	

	$sqla = "CREATE TABLE {$sqla_table_name} (

		id bigint(20) unsigned NOT NULL auto_increment ,

		uid varchar(64) NOT NULL,

		date bigint(10) unsigned NOT NULL, 

		type varchar(3) NOT NULL, 

		content mediumtext NOT NULL, 

		meta mediumtext NOT NULL,

		PRIMARY KEY  (id),

		UNIQUE KEY uid (uid), 

		KEY date (date), KEY type (type) 

		) $charset_collate;";



	# create logs table	

	$sqlb = "CREATE TABLE {$sqlb_table_name} (

		id bigint(20) unsigned NOT NULL auto_increment, 

		uid varchar(64) NOT NULL,

		date bigint(10) unsigned NOT NULL, 

		type varchar(10) NOT NULL, 

		msg mediumtext NOT NULL, 

		meta mediumtext NOT NULL, 

		PRIMARY KEY  (id), 

		UNIQUE KEY uid (uid), 

		KEY date (date), 

		KEY type (type)

		) $charset_collate;";



	# run sql

	# https://developer.wordpress.org/reference/functions/dbdelta/

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

	dbDelta( $sqla );

	dbDelta( $sqlb );

	

	# test if at least one table exists

	if (!$wpdb->get_var($wpdb->prepare("SHOW TABLES LIKE %s", $sqla_table_name)) === $sqla_table_name) {

		

		# log

		$err = 'An error occurred when trying to create the database tables';

		error_log($err);

		

		# alert

		if(is_admin()) {

			$notices = array($err);

			set_transient('codefonline_admin_notice_'.get_current_user_id(), json_encode($notices), 10);

		}

		

		# try again in 1 hour

		set_transient('codefonline_table_checker', true, HOUR_IN_SECONDS);

		

	} else {

		# success, but check again tomorrow

		set_transient('codefonline_table_checker', true, DAY_IN_SECONDS);

	}



}





# run during deactivation

register_deactivation_hook($codefonline_var_file, 'codefonline_plugin_deactivate');

function codefonline_plugin_deactivate() {

	

	# process cache settings

	codefonline_purge_static_files();



	global $wpdb;

	if(is_null($wpdb)) { return false; }

	

	# remove options and tables

	$wpdb->query("DELETE FROM {$wpdb->prefix}options WHERE option_name = 'codefonline_last_cache_update'");

	$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}codefonline_cache");

	$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}codefonline_logs");	



}



# run during uninstall

register_uninstall_hook($codefonline_var_file, 'codefonline_plugin_uninstall');	

function codefonline_plugin_uninstall() {

	

	# process cache settings

	codefonline_purge_static_files();

	

	global $wpdb;

	if(is_null($wpdb)) { return false; }

	

	# remove options and tables

	$wpdb->query("DELETE FROM {$wpdb->prefix}options WHERE option_name = 'codefonline_settings'");

	$wpdb->query("DELETE FROM {$wpdb->prefix}options WHERE option_name = 'codefonline_last_cache_update'");

	$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}codefonline_cache");

	$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}codefonline_logs");

	

}





# get all known roles

function codefonline_get_user_roles_checkboxes() {

	

	global $wp_roles, $codefonline_settings;

	$roles_list = array();

	if(is_object($wp_roles)) {

		$roles = (array) $wp_roles->get_names();

		foreach ($roles as $role=>$rname) {

			

			$roles_list[] = '<label for="codefonline_settings_minify_'.$role.'"><input name="codefonline_settings[minify]['.$role.']" type="checkbox" id="codefonline_settings_minify_'.$role.'" value="1" '. codefonline_get_settings_checkbox(codefonline_get_settings_value($codefonline_settings, 'minify', $role)).'> '.$rname.' </label><br />';

		

		}

	}

	

	# return

	if(!empty($roles_list)) { return implode(PHP_EOL, $roles_list); } else { return __( 'No roles detected!', 'codefusiononline' ); }



}

function codefonline_create_custom_table() {
    global $wpdb;

    $table_name = $wpdb->prefix . 'codefonline_urls';  // Define the table name
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE $table_name (
        id BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
        url VARCHAR(255) NOT NULL,
        critical_css_status ENUM('pending', 'processed', 'failed', 'done') DEFAULT 'pending',
        PRIMARY KEY (id),
        UNIQUE KEY url (url)
    ) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);
}
register_activation_hook($codefonline_var_file, 'codefonline_create_custom_table');

function codefonline_get_all_urls() {
    global $wpdb;

    // Get all posts
    $posts = get_posts(array(
        'numberposts' => -1,
        'post_type' => 'any',  // You can change this to target specific post types
        'post_status' => 'publish'
    ));

    // Get the homepage URL (this assumes the homepage is either a static page or a blog page)
    $home_url = get_home_url();

    // Insert homepage URL into the custom table if not already present
    $existing_home_url = $wpdb->get_var($wpdb->prepare("SELECT id FROM {$wpdb->prefix}codefonline_urls WHERE url = %s", $home_url));
    if (!$existing_home_url) {
        $wpdb->insert(
            $wpdb->prefix . 'codefonline_urls',
            array(
                'url' => $home_url,
                'critical_css_status' => 'pending'
            )
        );
    }

    // Get URLs for all posts and insert them into the custom table
    foreach ($posts as $post) {
        $url = get_permalink($post->ID); // Get the permalink (URL) for the post

        // Check if the URL is already in the table
        $existing_url = $wpdb->get_var($wpdb->prepare("SELECT id FROM {$wpdb->prefix}codefonline_urls WHERE url = %s", $url));

        if (!$existing_url) {
            // Insert URL into custom table with default 'pending' status
            $wpdb->insert(
                $wpdb->prefix . 'codefonline_urls',
                array(
                    'url' => $url,
                    'critical_css_status' => 'pending'
                )
            );
        }
    }
}


function codefonline_update_critical_css_status($url, $status) {
    global $wpdb;
    
    // Update the critical CSS status for the URL
    $wpdb->update(
        $wpdb->prefix . 'codefonline_urls',
        array('critical_css_status' => $status),
        array('url' => $url)
    );
}

// On plugin activation or specific cron job
function codefonline_on_activation() {
    codefonline_get_all_urls(); // Extract URLs and insert into custom table
}
register_activation_hook($codefonline_var_file, 'codefonline_on_activation');
function codefonline_critical_css_settings() {
    global $wpdb;

    // Get all URLs and statuses
    $urls = $wpdb->get_results("SELECT url, critical_css_status FROM {$wpdb->prefix}codefonline_urls");

    ?>
    <div class="wrap">
        <h1>Critical CSS Settings</h1>
        <table class="form-table">
            <thead>
                <tr>
                    <th scope="col">URL</th>
                    <th scope="col">Critical CSS Status</th>
                </tr>
            </thead>
            <tbody>
                <?php foreach ($urls as $url): ?>
                    <tr>
                        <td><?php echo esc_url($url->url); ?></td>
                        <td><?php echo esc_html(ucfirst($url->critical_css_status)); ?></td>
                    </tr>
                <?php endforeach; ?>
            </tbody>
        </table>
    </div>

    <div class="wrap">
        <h1>Create Critical CSS</h1>
        <button id="startButton" class="button-primary">Start CSS Processing</button>
        <div id="progressBar" style="width: 100%; background-color: #f3f3f3; border: 1px solid #ccc; height: 25px; margin-top: 10px; display: none;">
            <div id="progress" style="background: linear-gradient(135deg, rgba(255,255,255,0.3) 25%, transparent 25%, transparent 50%, rgba(255,255,255,0.3) 50%, rgba(255,255,255,0.3) 75%, transparent 75%, transparent); 
                             background-size: 30px 30px;
                             animation: progressAnimation 10s linear infinite, moveStripes 5s linear infinite;
                             height: 100%; width: 0%;
                             background-color: rgb(76, 175, 80);
                             "></div>
        </div>
        <div id="message"></div>
    </div>
    <style>


@keyframes moveStripes {
    0% { background-position: 0 0; }
    100% { background-position: 30px 30px; }
}
</style>
    <script type="text/javascript">
        jQuery(document).ready(function($){
            var urls = <?php echo json_encode($urls); ?>; // Get the URLs from PHP to JS
            var currentIndex = 0;
            $('#progress').css('width', '0%');

            function processNextUrl() {
                if (currentIndex < urls.length) {
                    var url = urls[currentIndex].url;
                    var status = urls[currentIndex].critical_css_status;

                    $('#progressBar').show();
                    
                    $('#message').append('<br>Processing URL: ' + url);
                    if(status != 'done'){
                        $.ajax({
                            url: ajaxurl,
                            method: 'POST',
                            data: {
                                action: 'process_critical_css',
                                url: url
                            },
                            success: function(response) {
                                if (response.success) {
                                    $('#message').append('<br>Critical CSS Processed for: ' + url);
                                    $('#progress').css('width', ((currentIndex + 1) / urls.length) * 100 + '%');
                                    currentIndex++;  // Move to the next URL
                                    processNextUrl(); // Process the next URL
                                } else {
                                    $('#message').append('<br>Error: ' + response.data.error);
                                    //$('#progress').css('width', '0%');
                                }
                            },
                            error: function() {
                                $('#message').append('<br>An error occurred.');
                                //$('#progress').css('width', '0%');
                            }
                        });
                    } else {
                        $('#message').append('<br>Critical CSS exist for: ' + url);
                        $('#progress').css('width', ((currentIndex + 1) / urls.length) * 100 + '%');
                        currentIndex++;  // Move to the next URL
                        processNextUrl(); // Process the next URL
                    }
                } else {
                    $('#message').append('<br>All URLs processed!');
                    $('#progress').css('width', '100%');
                }
            }

            $('#startButton').click(function() {
				let progressElement = document.getElementById("progress");
				let targetWidth = progressElement.style.width; // Read the width (e.g., "82%")

				// Reset width to 0% to animate properly
				progressElement.style.width = "0%";

				// Use setTimeout to animate dynamically
				//setTimeout(() => {
					progressElement.style.transition = "width 10s linear"; // Apply smooth transition
					progressElement.style.width = targetWidth; // Animate to the real width
				//}, 100); // Small delay to ensure it starts from 0%

				// Keep the moving stripes animation
				progressElement.style.animation = "moveStripes 5s linear infinite";
                currentIndex = 0;  // Reset the index
                processNextUrl();   // Start processing the URLs
            });
        });
    </script>
    <?php
}


function codefonline_admin_styles() {
    echo '<style>
        .form-table th {
            width: 30%;
            padding: 10px;
            text-align: left;
        }
        .form-table td {
            padding: 10px;
        }
    </style>';
}
add_action('admin_head', 'codefonline_admin_styles');

function normalize_url($url) {
    $url = esc_url_raw($url);
    $parsed_url = parse_url($url);

    // Normalize scheme and host
    $scheme = isset($parsed_url['scheme']) ? $parsed_url['scheme'] . '://' : '';
    $host = isset($parsed_url['host']) ? $parsed_url['host'] : '';
    $path = isset($parsed_url['path']) ? rtrim($parsed_url['path'], '/') : ''; // Remove trailing slash
    $query = isset($parsed_url['query']) ? '?' . $parsed_url['query'] : '';

    return $scheme . $host . $path . $query;
}

function handle_critical_css_processing() {
    global $codefonline_settings;
    if (isset($_POST['url'])) {
        $url = sanitize_text_field($_POST['url']);
        codefonline_update_critical_css_status($url, 'processed');
        // Send the remote POST request to the external URL
        $response = wp_remote_post('https://codefusiononline.com/critical-css', array(
            'method'    => 'GET',
			'timeout' => 60,
			'sslverify' => false,
            'body'      => array(
                'url' => $url,
                'apikey' => codefonline_get_settings_value($codefonline_settings, 'site', 'key'), // Replace with your actual API key
            ),
        ));
        
        if (is_wp_error($response)) {
            codefonline_update_critical_css_status($url, 'failed');
            wp_send_json_error(array('error' => 'Failed to connect to the external server.'));
        }

        $response_body = wp_remote_retrieve_body($response);
        
        if (empty($response_body)) {
            codefonline_update_critical_css_status($url, 'failed');
            wp_send_json_error(array('error' => 'Empty response from server.'));
        }
        
        // Check if the response contains the "Invalid API key" error
        $response_data = json_decode($response_body, true);
        if (isset($response_data['error']) && $response_data['error'] === 'Invalid API key') {
            codefonline_update_critical_css_status($url, 'failed');
            wp_send_json_error(array('error' => 'Invalid API key.'));
        }
        
        if (isset($response_data['data'])){
            // Assuming the response contains the critical CSS as plain text
            $critical_css = $response_data['data'];
    
            // Create a directory to store the CSS file
            $cache_dir = WP_CONTENT_DIR . '/cache/codefonline/critical-css';
            if (!is_dir($cache_dir)) {
                mkdir($cache_dir, 0755, true);
            }
            
            codefonline_update_critical_css_status($url, 'done');
            
            $url = normalize_url(sanitize_text_field($_POST['url']));
            // Create a filename from the URL
            $file_name = md5($url) . '.css';
            $file_path = $cache_dir . '/' . $file_name;
            
            // Write the CSS to the file
            file_put_contents($file_path, $critical_css);
        } else {
            codefonline_update_critical_css_status($url, 'failed');
            wp_send_json_error($response_data);
        }
        
        

        // Add the critical CSS to the <head> section of every page
        //add_action('wp_head', function() use ($file_path) {
        //    echo '<link rel="stylesheet" href="' . content_url('/cache/codefonline/critical-css/' . basename($file_path)) . '" />';
        //});

        wp_send_json_success();
    } else {
        wp_send_json_error(array('error' => 'URL is missing.'));
    }
}

add_action('wp_ajax_process_critical_css', 'handle_critical_css_processing'); // For logged-in users
add_action('wp_ajax_nopriv_process_critical_css', 'handle_critical_css_processing'); // For non-logged-in users
